# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: CasperMessage.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2
from . import RhoTypes_pb2 as RhoTypes__pb2
from . import Either_pb2 as Either__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='CasperMessage.proto',
  package='coop.rchain.casper.protocol',
  syntax='proto3',
  serialized_options=_b('\342?=\n\033coop.rchain.casper.protocol\020\001\"\032sealed trait CasperMessage(\001'),
  serialized_pb=_b('\n\x13\x43\x61sperMessage.proto\x12\x1b\x63oop.rchain.casper.protocol\x1a\x1bgoogle/protobuf/empty.proto\x1a\x15scalapb/scalapb.proto\x1a\x0eRhoTypes.proto\x1a\x0c\x45ither.proto\"3\n\x0fHasBlockRequest\x12\x0c\n\x04hash\x18\x01 \x01(\x0c:\x12\xe2?\x0f\n\rCasperMessage\",\n\x08HasBlock\x12\x0c\n\x04hash\x18\x01 \x01(\x0c:\x12\xe2?\x0f\n\rCasperMessage\"0\n\x0c\x42lockRequest\x12\x0c\n\x04hash\x18\x01 \x01(\x0c:\x12\xe2?\x0f\n\rCasperMessage\"*\n\x14\x46orkChoiceTipRequest:\x12\xe2?\x0f\n\rCasperMessage\"h\n\x16\x41pprovedBlockCandidate\x12\x38\n\x05\x62lock\x18\x01 \x01(\x0b\x32).coop.rchain.casper.protocol.BlockMessage\x12\x14\n\x0crequiredSigs\x18\x02 \x01(\x05\"\x92\x01\n\x0fUnapprovedBlock\x12\x46\n\tcandidate\x18\x01 \x01(\x0b\x32\x33.coop.rchain.casper.protocol.ApprovedBlockCandidate\x12\x11\n\ttimestamp\x18\x02 \x01(\x03\x12\x10\n\x08\x64uration\x18\x03 \x01(\x03:\x12\xe2?\x0f\n\rCasperMessage\">\n\tSignature\x12\x11\n\tpublicKey\x18\x01 \x01(\x0c\x12\x11\n\talgorithm\x18\x02 \x01(\t\x12\x0b\n\x03sig\x18\x03 \x01(\x0c\"\xa0\x01\n\rBlockApproval\x12\x46\n\tcandidate\x18\x01 \x01(\x0b\x32\x33.coop.rchain.casper.protocol.ApprovedBlockCandidate\x12\x33\n\x03sig\x18\x02 \x01(\x0b\x32&.coop.rchain.casper.protocol.Signature:\x12\xe2?\x0f\n\rCasperMessage\"\xa1\x01\n\rApprovedBlock\x12\x46\n\tcandidate\x18\x01 \x01(\x0b\x32\x33.coop.rchain.casper.protocol.ApprovedBlockCandidate\x12\x34\n\x04sigs\x18\x02 \x03(\x0b\x32&.coop.rchain.casper.protocol.Signature:\x12\xe2?\x0f\n\rCasperMessage\">\n\x14\x41pprovedBlockRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t:\x12\xe2?\x0f\n\rCasperMessage\"Y\n\x18NoApprovedBlockAvailable\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x15\n\rnodeIdentifer\x18\x02 \x01(\t:\x12\xe2?\x0f\n\rCasperMessage\"\xc7\x02\n\x0c\x42lockMessage\x12\x11\n\tblockHash\x18\x01 \x01(\x0c\x12\x33\n\x06header\x18\x02 \x01(\x0b\x32#.coop.rchain.casper.protocol.Header\x12/\n\x04\x62ody\x18\x03 \x01(\x0b\x32!.coop.rchain.casper.protocol.Body\x12\x42\n\x0ejustifications\x18\x04 \x03(\x0b\x32*.coop.rchain.casper.protocol.Justification\x12\x0e\n\x06sender\x18\x05 \x01(\x0c\x12\x0e\n\x06seqNum\x18\x06 \x01(\x05\x12\x0b\n\x03sig\x18\x07 \x01(\x0c\x12\x14\n\x0csigAlgorithm\x18\x08 \x01(\t\x12\x0f\n\x07shardId\x18\t \x01(\t\x12\x12\n\nextraBytes\x18\n \x01(\x0c:\x12\xe2?\x0f\n\rCasperMessage\"\xfb\x02\n\x15\x42lockMetadataInternal\x12\x11\n\tblockHash\x18\x01 \x01(\x0c\x12/\n\x07parents\x18\x02 \x03(\x0c\x42\x1e\xe2?\x1b\x1a\x19\x63ollection.immutable.List\x12\x0e\n\x06sender\x18\x03 \x01(\x0c\x12\x62\n\x0ejustifications\x18\x04 \x03(\x0b\x32*.coop.rchain.casper.protocol.JustificationB\x1e\xe2?\x1b\x1a\x19\x63ollection.immutable.List\x12P\n\x05\x62onds\x18\x05 \x03(\x0b\x32!.coop.rchain.casper.protocol.BondB\x1e\xe2?\x1b\x1a\x19\x63ollection.immutable.List\x12\x10\n\x08\x62lockNum\x18\x06 \x01(\x03\x12\x0e\n\x06seqNum\x18\x07 \x01(\x05\x12\x0f\n\x07invalid\x18\x08 \x01(\x08:%\xe2?\"\" coop.rchain.models.BlockMetadata\"\x9a\x01\n\x06Header\x12\x17\n\x0fparentsHashList\x18\x01 \x03(\x0c\x12\x15\n\rpostStateHash\x18\x02 \x01(\x0c\x12\x13\n\x0b\x64\x65ploysHash\x18\x03 \x01(\x0c\x12\x11\n\ttimestamp\x18\x05 \x01(\x03\x12\x0f\n\x07version\x18\x06 \x01(\x03\x12\x13\n\x0b\x64\x65ployCount\x18\x07 \x01(\x05\x12\x12\n\nextraBytes\x18\x08 \x01(\x0c\"\xa7\x01\n\nDeployData\x12\x10\n\x08\x64\x65ployer\x18\x01 \x01(\x0c\x12\x0c\n\x04term\x18\x02 \x01(\t\x12\x11\n\ttimestamp\x18\x03 \x01(\x03\x12\x0b\n\x03sig\x18\x04 \x01(\x0c\x12\x14\n\x0csigAlgorithm\x18\x05 \x01(\t\x12\x11\n\tphloPrice\x18\x07 \x01(\x03\x12\x11\n\tphloLimit\x18\x08 \x01(\x03\x12\x1d\n\x15validAfterBlockNumber\x18\n \x01(\x03\"\xe0\x01\n\x0fProcessedDeploy\x12\x37\n\x06\x64\x65ploy\x18\x01 \x01(\x0b\x32\'.coop.rchain.casper.protocol.DeployData\x12\x14\n\x04\x63ost\x18\x02 \x01(\x0b\x32\x06.PCost\x12\x35\n\tdeployLog\x18\x03 \x03(\x0b\x32\".coop.rchain.casper.protocol.Event\x12\x36\n\npaymentLog\x18\x04 \x03(\x0b\x32\".coop.rchain.casper.protocol.Event\x12\x0f\n\x07\x65rrored\x18\x05 \x01(\x08\"\x92\x01\n\x04\x42ody\x12\x37\n\x05state\x18\x01 \x01(\x0b\x32(.coop.rchain.casper.protocol.RChainState\x12=\n\x07\x64\x65ploys\x18\x02 \x03(\x0b\x32,.coop.rchain.casper.protocol.ProcessedDeploy\x12\x12\n\nextraBytes\x18\x03 \x01(\x0c\";\n\rJustification\x12\x11\n\tvalidator\x18\x01 \x01(\x0c\x12\x17\n\x0flatestBlockHash\x18\x02 \x01(\x0c\"\x81\x01\n\x0bRChainState\x12\x14\n\x0cpreStateHash\x18\x01 \x01(\x0c\x12\x15\n\rpostStateHash\x18\x02 \x01(\x0c\x12\x30\n\x05\x62onds\x18\x03 \x03(\x0b\x32!.coop.rchain.casper.protocol.Bond\x12\x13\n\x0b\x62lockNumber\x18\x04 \x01(\x03\"\xcd\x01\n\x05\x45vent\x12<\n\x07produce\x18\x01 \x01(\x0b\x32).coop.rchain.casper.protocol.ProduceEventH\x00\x12<\n\x07\x63onsume\x18\x02 \x01(\x0b\x32).coop.rchain.casper.protocol.ConsumeEventH\x00\x12\x36\n\x04\x63omm\x18\x03 \x01(\x0b\x32&.coop.rchain.casper.protocol.CommEventH\x00\x42\x10\n\x0e\x65vent_instance\"J\n\x0cProduceEvent\x12\x14\n\x0c\x63hannelsHash\x18\x01 \x01(\x0c\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12\x16\n\x0esequenceNumber\x18\x03 \x01(\x05\"L\n\x0c\x43onsumeEvent\x12\x16\n\x0e\x63hannelsHashes\x18\x01 \x03(\x0c\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12\x16\n\x0esequenceNumber\x18\x03 \x01(\x05\"\x84\x01\n\tCommEvent\x12:\n\x07\x63onsume\x18\x01 \x01(\x0b\x32).coop.rchain.casper.protocol.ConsumeEvent\x12;\n\x08produces\x18\x02 \x03(\x0b\x32).coop.rchain.casper.protocol.ProduceEvent\"(\n\x04\x42ond\x12\x11\n\tvalidator\x18\x01 \x01(\x0c\x12\r\n\x05stake\x18\x02 \x01(\x03\x42@\xe2?=\n\x1b\x63oop.rchain.casper.protocol\x10\x01\"\x1asealed trait CasperMessage(\x01\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,scalapb_dot_scalapb__pb2.DESCRIPTOR,RhoTypes__pb2.DESCRIPTOR,Either__pb2.DESCRIPTOR,])




_HASBLOCKREQUEST = _descriptor.Descriptor(
  name='HasBlockRequest',
  full_name='coop.rchain.casper.protocol.HasBlockRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='coop.rchain.casper.protocol.HasBlockRequest.hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=134,
  serialized_end=185,
)


_HASBLOCK = _descriptor.Descriptor(
  name='HasBlock',
  full_name='coop.rchain.casper.protocol.HasBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='coop.rchain.casper.protocol.HasBlock.hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=187,
  serialized_end=231,
)


_BLOCKREQUEST = _descriptor.Descriptor(
  name='BlockRequest',
  full_name='coop.rchain.casper.protocol.BlockRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='coop.rchain.casper.protocol.BlockRequest.hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=233,
  serialized_end=281,
)


_FORKCHOICETIPREQUEST = _descriptor.Descriptor(
  name='ForkChoiceTipRequest',
  full_name='coop.rchain.casper.protocol.ForkChoiceTipRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=283,
  serialized_end=325,
)


_APPROVEDBLOCKCANDIDATE = _descriptor.Descriptor(
  name='ApprovedBlockCandidate',
  full_name='coop.rchain.casper.protocol.ApprovedBlockCandidate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block', full_name='coop.rchain.casper.protocol.ApprovedBlockCandidate.block', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='requiredSigs', full_name='coop.rchain.casper.protocol.ApprovedBlockCandidate.requiredSigs', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=327,
  serialized_end=431,
)


_UNAPPROVEDBLOCK = _descriptor.Descriptor(
  name='UnapprovedBlock',
  full_name='coop.rchain.casper.protocol.UnapprovedBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='candidate', full_name='coop.rchain.casper.protocol.UnapprovedBlock.candidate', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='coop.rchain.casper.protocol.UnapprovedBlock.timestamp', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='duration', full_name='coop.rchain.casper.protocol.UnapprovedBlock.duration', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=434,
  serialized_end=580,
)


_SIGNATURE = _descriptor.Descriptor(
  name='Signature',
  full_name='coop.rchain.casper.protocol.Signature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='publicKey', full_name='coop.rchain.casper.protocol.Signature.publicKey', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='algorithm', full_name='coop.rchain.casper.protocol.Signature.algorithm', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sig', full_name='coop.rchain.casper.protocol.Signature.sig', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=582,
  serialized_end=644,
)


_BLOCKAPPROVAL = _descriptor.Descriptor(
  name='BlockApproval',
  full_name='coop.rchain.casper.protocol.BlockApproval',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='candidate', full_name='coop.rchain.casper.protocol.BlockApproval.candidate', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sig', full_name='coop.rchain.casper.protocol.BlockApproval.sig', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=647,
  serialized_end=807,
)


_APPROVEDBLOCK = _descriptor.Descriptor(
  name='ApprovedBlock',
  full_name='coop.rchain.casper.protocol.ApprovedBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='candidate', full_name='coop.rchain.casper.protocol.ApprovedBlock.candidate', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sigs', full_name='coop.rchain.casper.protocol.ApprovedBlock.sigs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=810,
  serialized_end=971,
)


_APPROVEDBLOCKREQUEST = _descriptor.Descriptor(
  name='ApprovedBlockRequest',
  full_name='coop.rchain.casper.protocol.ApprovedBlockRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='identifier', full_name='coop.rchain.casper.protocol.ApprovedBlockRequest.identifier', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=973,
  serialized_end=1035,
)


_NOAPPROVEDBLOCKAVAILABLE = _descriptor.Descriptor(
  name='NoApprovedBlockAvailable',
  full_name='coop.rchain.casper.protocol.NoApprovedBlockAvailable',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='identifier', full_name='coop.rchain.casper.protocol.NoApprovedBlockAvailable.identifier', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nodeIdentifer', full_name='coop.rchain.casper.protocol.NoApprovedBlockAvailable.nodeIdentifer', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1037,
  serialized_end=1126,
)


_BLOCKMESSAGE = _descriptor.Descriptor(
  name='BlockMessage',
  full_name='coop.rchain.casper.protocol.BlockMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blockHash', full_name='coop.rchain.casper.protocol.BlockMessage.blockHash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header', full_name='coop.rchain.casper.protocol.BlockMessage.header', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='body', full_name='coop.rchain.casper.protocol.BlockMessage.body', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='justifications', full_name='coop.rchain.casper.protocol.BlockMessage.justifications', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sender', full_name='coop.rchain.casper.protocol.BlockMessage.sender', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqNum', full_name='coop.rchain.casper.protocol.BlockMessage.seqNum', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sig', full_name='coop.rchain.casper.protocol.BlockMessage.sig', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sigAlgorithm', full_name='coop.rchain.casper.protocol.BlockMessage.sigAlgorithm', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shardId', full_name='coop.rchain.casper.protocol.BlockMessage.shardId', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extraBytes', full_name='coop.rchain.casper.protocol.BlockMessage.extraBytes', index=9,
      number=10, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\017\n\rCasperMessage'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1129,
  serialized_end=1456,
)


_BLOCKMETADATAINTERNAL = _descriptor.Descriptor(
  name='BlockMetadataInternal',
  full_name='coop.rchain.casper.protocol.BlockMetadataInternal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blockHash', full_name='coop.rchain.casper.protocol.BlockMetadataInternal.blockHash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parents', full_name='coop.rchain.casper.protocol.BlockMetadataInternal.parents', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\342?\033\032\031collection.immutable.List'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sender', full_name='coop.rchain.casper.protocol.BlockMetadataInternal.sender', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='justifications', full_name='coop.rchain.casper.protocol.BlockMetadataInternal.justifications', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\342?\033\032\031collection.immutable.List'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bonds', full_name='coop.rchain.casper.protocol.BlockMetadataInternal.bonds', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\342?\033\032\031collection.immutable.List'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blockNum', full_name='coop.rchain.casper.protocol.BlockMetadataInternal.blockNum', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqNum', full_name='coop.rchain.casper.protocol.BlockMetadataInternal.seqNum', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='invalid', full_name='coop.rchain.casper.protocol.BlockMetadataInternal.invalid', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('\342?\"\" coop.rchain.models.BlockMetadata'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1459,
  serialized_end=1838,
)


_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='coop.rchain.casper.protocol.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='parentsHashList', full_name='coop.rchain.casper.protocol.Header.parentsHashList', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='postStateHash', full_name='coop.rchain.casper.protocol.Header.postStateHash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deploysHash', full_name='coop.rchain.casper.protocol.Header.deploysHash', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='coop.rchain.casper.protocol.Header.timestamp', index=3,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='coop.rchain.casper.protocol.Header.version', index=4,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deployCount', full_name='coop.rchain.casper.protocol.Header.deployCount', index=5,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extraBytes', full_name='coop.rchain.casper.protocol.Header.extraBytes', index=6,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1841,
  serialized_end=1995,
)


_DEPLOYDATA = _descriptor.Descriptor(
  name='DeployData',
  full_name='coop.rchain.casper.protocol.DeployData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployer', full_name='coop.rchain.casper.protocol.DeployData.deployer', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='term', full_name='coop.rchain.casper.protocol.DeployData.term', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='coop.rchain.casper.protocol.DeployData.timestamp', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sig', full_name='coop.rchain.casper.protocol.DeployData.sig', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sigAlgorithm', full_name='coop.rchain.casper.protocol.DeployData.sigAlgorithm', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='phloPrice', full_name='coop.rchain.casper.protocol.DeployData.phloPrice', index=5,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='phloLimit', full_name='coop.rchain.casper.protocol.DeployData.phloLimit', index=6,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='validAfterBlockNumber', full_name='coop.rchain.casper.protocol.DeployData.validAfterBlockNumber', index=7,
      number=10, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1998,
  serialized_end=2165,
)


_PROCESSEDDEPLOY = _descriptor.Descriptor(
  name='ProcessedDeploy',
  full_name='coop.rchain.casper.protocol.ProcessedDeploy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy', full_name='coop.rchain.casper.protocol.ProcessedDeploy.deploy', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cost', full_name='coop.rchain.casper.protocol.ProcessedDeploy.cost', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deployLog', full_name='coop.rchain.casper.protocol.ProcessedDeploy.deployLog', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='paymentLog', full_name='coop.rchain.casper.protocol.ProcessedDeploy.paymentLog', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errored', full_name='coop.rchain.casper.protocol.ProcessedDeploy.errored', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2168,
  serialized_end=2392,
)


_BODY = _descriptor.Descriptor(
  name='Body',
  full_name='coop.rchain.casper.protocol.Body',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='coop.rchain.casper.protocol.Body.state', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deploys', full_name='coop.rchain.casper.protocol.Body.deploys', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extraBytes', full_name='coop.rchain.casper.protocol.Body.extraBytes', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2395,
  serialized_end=2541,
)


_JUSTIFICATION = _descriptor.Descriptor(
  name='Justification',
  full_name='coop.rchain.casper.protocol.Justification',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='validator', full_name='coop.rchain.casper.protocol.Justification.validator', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='latestBlockHash', full_name='coop.rchain.casper.protocol.Justification.latestBlockHash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2543,
  serialized_end=2602,
)


_RCHAINSTATE = _descriptor.Descriptor(
  name='RChainState',
  full_name='coop.rchain.casper.protocol.RChainState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='preStateHash', full_name='coop.rchain.casper.protocol.RChainState.preStateHash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='postStateHash', full_name='coop.rchain.casper.protocol.RChainState.postStateHash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bonds', full_name='coop.rchain.casper.protocol.RChainState.bonds', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blockNumber', full_name='coop.rchain.casper.protocol.RChainState.blockNumber', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2605,
  serialized_end=2734,
)


_EVENT = _descriptor.Descriptor(
  name='Event',
  full_name='coop.rchain.casper.protocol.Event',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='produce', full_name='coop.rchain.casper.protocol.Event.produce', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='consume', full_name='coop.rchain.casper.protocol.Event.consume', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='comm', full_name='coop.rchain.casper.protocol.Event.comm', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='event_instance', full_name='coop.rchain.casper.protocol.Event.event_instance',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=2737,
  serialized_end=2942,
)


_PRODUCEEVENT = _descriptor.Descriptor(
  name='ProduceEvent',
  full_name='coop.rchain.casper.protocol.ProduceEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channelsHash', full_name='coop.rchain.casper.protocol.ProduceEvent.channelsHash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hash', full_name='coop.rchain.casper.protocol.ProduceEvent.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sequenceNumber', full_name='coop.rchain.casper.protocol.ProduceEvent.sequenceNumber', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2944,
  serialized_end=3018,
)


_CONSUMEEVENT = _descriptor.Descriptor(
  name='ConsumeEvent',
  full_name='coop.rchain.casper.protocol.ConsumeEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channelsHashes', full_name='coop.rchain.casper.protocol.ConsumeEvent.channelsHashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hash', full_name='coop.rchain.casper.protocol.ConsumeEvent.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sequenceNumber', full_name='coop.rchain.casper.protocol.ConsumeEvent.sequenceNumber', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3020,
  serialized_end=3096,
)


_COMMEVENT = _descriptor.Descriptor(
  name='CommEvent',
  full_name='coop.rchain.casper.protocol.CommEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='consume', full_name='coop.rchain.casper.protocol.CommEvent.consume', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='produces', full_name='coop.rchain.casper.protocol.CommEvent.produces', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3099,
  serialized_end=3231,
)


_BOND = _descriptor.Descriptor(
  name='Bond',
  full_name='coop.rchain.casper.protocol.Bond',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='validator', full_name='coop.rchain.casper.protocol.Bond.validator', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stake', full_name='coop.rchain.casper.protocol.Bond.stake', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3233,
  serialized_end=3273,
)

_APPROVEDBLOCKCANDIDATE.fields_by_name['block'].message_type = _BLOCKMESSAGE
_UNAPPROVEDBLOCK.fields_by_name['candidate'].message_type = _APPROVEDBLOCKCANDIDATE
_BLOCKAPPROVAL.fields_by_name['candidate'].message_type = _APPROVEDBLOCKCANDIDATE
_BLOCKAPPROVAL.fields_by_name['sig'].message_type = _SIGNATURE
_APPROVEDBLOCK.fields_by_name['candidate'].message_type = _APPROVEDBLOCKCANDIDATE
_APPROVEDBLOCK.fields_by_name['sigs'].message_type = _SIGNATURE
_BLOCKMESSAGE.fields_by_name['header'].message_type = _HEADER
_BLOCKMESSAGE.fields_by_name['body'].message_type = _BODY
_BLOCKMESSAGE.fields_by_name['justifications'].message_type = _JUSTIFICATION
_BLOCKMETADATAINTERNAL.fields_by_name['justifications'].message_type = _JUSTIFICATION
_BLOCKMETADATAINTERNAL.fields_by_name['bonds'].message_type = _BOND
_PROCESSEDDEPLOY.fields_by_name['deploy'].message_type = _DEPLOYDATA
_PROCESSEDDEPLOY.fields_by_name['cost'].message_type = RhoTypes__pb2._PCOST
_PROCESSEDDEPLOY.fields_by_name['deployLog'].message_type = _EVENT
_PROCESSEDDEPLOY.fields_by_name['paymentLog'].message_type = _EVENT
_BODY.fields_by_name['state'].message_type = _RCHAINSTATE
_BODY.fields_by_name['deploys'].message_type = _PROCESSEDDEPLOY
_RCHAINSTATE.fields_by_name['bonds'].message_type = _BOND
_EVENT.fields_by_name['produce'].message_type = _PRODUCEEVENT
_EVENT.fields_by_name['consume'].message_type = _CONSUMEEVENT
_EVENT.fields_by_name['comm'].message_type = _COMMEVENT
_EVENT.oneofs_by_name['event_instance'].fields.append(
  _EVENT.fields_by_name['produce'])
_EVENT.fields_by_name['produce'].containing_oneof = _EVENT.oneofs_by_name['event_instance']
_EVENT.oneofs_by_name['event_instance'].fields.append(
  _EVENT.fields_by_name['consume'])
_EVENT.fields_by_name['consume'].containing_oneof = _EVENT.oneofs_by_name['event_instance']
_EVENT.oneofs_by_name['event_instance'].fields.append(
  _EVENT.fields_by_name['comm'])
_EVENT.fields_by_name['comm'].containing_oneof = _EVENT.oneofs_by_name['event_instance']
_COMMEVENT.fields_by_name['consume'].message_type = _CONSUMEEVENT
_COMMEVENT.fields_by_name['produces'].message_type = _PRODUCEEVENT
DESCRIPTOR.message_types_by_name['HasBlockRequest'] = _HASBLOCKREQUEST
DESCRIPTOR.message_types_by_name['HasBlock'] = _HASBLOCK
DESCRIPTOR.message_types_by_name['BlockRequest'] = _BLOCKREQUEST
DESCRIPTOR.message_types_by_name['ForkChoiceTipRequest'] = _FORKCHOICETIPREQUEST
DESCRIPTOR.message_types_by_name['ApprovedBlockCandidate'] = _APPROVEDBLOCKCANDIDATE
DESCRIPTOR.message_types_by_name['UnapprovedBlock'] = _UNAPPROVEDBLOCK
DESCRIPTOR.message_types_by_name['Signature'] = _SIGNATURE
DESCRIPTOR.message_types_by_name['BlockApproval'] = _BLOCKAPPROVAL
DESCRIPTOR.message_types_by_name['ApprovedBlock'] = _APPROVEDBLOCK
DESCRIPTOR.message_types_by_name['ApprovedBlockRequest'] = _APPROVEDBLOCKREQUEST
DESCRIPTOR.message_types_by_name['NoApprovedBlockAvailable'] = _NOAPPROVEDBLOCKAVAILABLE
DESCRIPTOR.message_types_by_name['BlockMessage'] = _BLOCKMESSAGE
DESCRIPTOR.message_types_by_name['BlockMetadataInternal'] = _BLOCKMETADATAINTERNAL
DESCRIPTOR.message_types_by_name['Header'] = _HEADER
DESCRIPTOR.message_types_by_name['DeployData'] = _DEPLOYDATA
DESCRIPTOR.message_types_by_name['ProcessedDeploy'] = _PROCESSEDDEPLOY
DESCRIPTOR.message_types_by_name['Body'] = _BODY
DESCRIPTOR.message_types_by_name['Justification'] = _JUSTIFICATION
DESCRIPTOR.message_types_by_name['RChainState'] = _RCHAINSTATE
DESCRIPTOR.message_types_by_name['Event'] = _EVENT
DESCRIPTOR.message_types_by_name['ProduceEvent'] = _PRODUCEEVENT
DESCRIPTOR.message_types_by_name['ConsumeEvent'] = _CONSUMEEVENT
DESCRIPTOR.message_types_by_name['CommEvent'] = _COMMEVENT
DESCRIPTOR.message_types_by_name['Bond'] = _BOND
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HasBlockRequest = _reflection.GeneratedProtocolMessageType('HasBlockRequest', (_message.Message,), {
  'DESCRIPTOR' : _HASBLOCKREQUEST,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.HasBlockRequest)
  })
_sym_db.RegisterMessage(HasBlockRequest)

HasBlock = _reflection.GeneratedProtocolMessageType('HasBlock', (_message.Message,), {
  'DESCRIPTOR' : _HASBLOCK,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.HasBlock)
  })
_sym_db.RegisterMessage(HasBlock)

BlockRequest = _reflection.GeneratedProtocolMessageType('BlockRequest', (_message.Message,), {
  'DESCRIPTOR' : _BLOCKREQUEST,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.BlockRequest)
  })
_sym_db.RegisterMessage(BlockRequest)

ForkChoiceTipRequest = _reflection.GeneratedProtocolMessageType('ForkChoiceTipRequest', (_message.Message,), {
  'DESCRIPTOR' : _FORKCHOICETIPREQUEST,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.ForkChoiceTipRequest)
  })
_sym_db.RegisterMessage(ForkChoiceTipRequest)

ApprovedBlockCandidate = _reflection.GeneratedProtocolMessageType('ApprovedBlockCandidate', (_message.Message,), {
  'DESCRIPTOR' : _APPROVEDBLOCKCANDIDATE,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.ApprovedBlockCandidate)
  })
_sym_db.RegisterMessage(ApprovedBlockCandidate)

UnapprovedBlock = _reflection.GeneratedProtocolMessageType('UnapprovedBlock', (_message.Message,), {
  'DESCRIPTOR' : _UNAPPROVEDBLOCK,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.UnapprovedBlock)
  })
_sym_db.RegisterMessage(UnapprovedBlock)

Signature = _reflection.GeneratedProtocolMessageType('Signature', (_message.Message,), {
  'DESCRIPTOR' : _SIGNATURE,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.Signature)
  })
_sym_db.RegisterMessage(Signature)

BlockApproval = _reflection.GeneratedProtocolMessageType('BlockApproval', (_message.Message,), {
  'DESCRIPTOR' : _BLOCKAPPROVAL,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.BlockApproval)
  })
_sym_db.RegisterMessage(BlockApproval)

ApprovedBlock = _reflection.GeneratedProtocolMessageType('ApprovedBlock', (_message.Message,), {
  'DESCRIPTOR' : _APPROVEDBLOCK,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.ApprovedBlock)
  })
_sym_db.RegisterMessage(ApprovedBlock)

ApprovedBlockRequest = _reflection.GeneratedProtocolMessageType('ApprovedBlockRequest', (_message.Message,), {
  'DESCRIPTOR' : _APPROVEDBLOCKREQUEST,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.ApprovedBlockRequest)
  })
_sym_db.RegisterMessage(ApprovedBlockRequest)

NoApprovedBlockAvailable = _reflection.GeneratedProtocolMessageType('NoApprovedBlockAvailable', (_message.Message,), {
  'DESCRIPTOR' : _NOAPPROVEDBLOCKAVAILABLE,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.NoApprovedBlockAvailable)
  })
_sym_db.RegisterMessage(NoApprovedBlockAvailable)

BlockMessage = _reflection.GeneratedProtocolMessageType('BlockMessage', (_message.Message,), {
  'DESCRIPTOR' : _BLOCKMESSAGE,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.BlockMessage)
  })
_sym_db.RegisterMessage(BlockMessage)

BlockMetadataInternal = _reflection.GeneratedProtocolMessageType('BlockMetadataInternal', (_message.Message,), {
  'DESCRIPTOR' : _BLOCKMETADATAINTERNAL,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.BlockMetadataInternal)
  })
_sym_db.RegisterMessage(BlockMetadataInternal)

Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
  'DESCRIPTOR' : _HEADER,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.Header)
  })
_sym_db.RegisterMessage(Header)

DeployData = _reflection.GeneratedProtocolMessageType('DeployData', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYDATA,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.DeployData)
  })
_sym_db.RegisterMessage(DeployData)

ProcessedDeploy = _reflection.GeneratedProtocolMessageType('ProcessedDeploy', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSEDDEPLOY,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.ProcessedDeploy)
  })
_sym_db.RegisterMessage(ProcessedDeploy)

Body = _reflection.GeneratedProtocolMessageType('Body', (_message.Message,), {
  'DESCRIPTOR' : _BODY,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.Body)
  })
_sym_db.RegisterMessage(Body)

Justification = _reflection.GeneratedProtocolMessageType('Justification', (_message.Message,), {
  'DESCRIPTOR' : _JUSTIFICATION,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.Justification)
  })
_sym_db.RegisterMessage(Justification)

RChainState = _reflection.GeneratedProtocolMessageType('RChainState', (_message.Message,), {
  'DESCRIPTOR' : _RCHAINSTATE,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.RChainState)
  })
_sym_db.RegisterMessage(RChainState)

Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,), {
  'DESCRIPTOR' : _EVENT,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.Event)
  })
_sym_db.RegisterMessage(Event)

ProduceEvent = _reflection.GeneratedProtocolMessageType('ProduceEvent', (_message.Message,), {
  'DESCRIPTOR' : _PRODUCEEVENT,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.ProduceEvent)
  })
_sym_db.RegisterMessage(ProduceEvent)

ConsumeEvent = _reflection.GeneratedProtocolMessageType('ConsumeEvent', (_message.Message,), {
  'DESCRIPTOR' : _CONSUMEEVENT,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.ConsumeEvent)
  })
_sym_db.RegisterMessage(ConsumeEvent)

CommEvent = _reflection.GeneratedProtocolMessageType('CommEvent', (_message.Message,), {
  'DESCRIPTOR' : _COMMEVENT,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.CommEvent)
  })
_sym_db.RegisterMessage(CommEvent)

Bond = _reflection.GeneratedProtocolMessageType('Bond', (_message.Message,), {
  'DESCRIPTOR' : _BOND,
  '__module__' : 'CasperMessage_pb2'
  # @@protoc_insertion_point(class_scope:coop.rchain.casper.protocol.Bond)
  })
_sym_db.RegisterMessage(Bond)


DESCRIPTOR._options = None
_HASBLOCKREQUEST._options = None
_HASBLOCK._options = None
_BLOCKREQUEST._options = None
_FORKCHOICETIPREQUEST._options = None
_UNAPPROVEDBLOCK._options = None
_BLOCKAPPROVAL._options = None
_APPROVEDBLOCK._options = None
_APPROVEDBLOCKREQUEST._options = None
_NOAPPROVEDBLOCKAVAILABLE._options = None
_BLOCKMESSAGE._options = None
_BLOCKMETADATAINTERNAL.fields_by_name['parents']._options = None
_BLOCKMETADATAINTERNAL.fields_by_name['justifications']._options = None
_BLOCKMETADATAINTERNAL.fields_by_name['bonds']._options = None
_BLOCKMETADATAINTERNAL._options = None
# @@protoc_insertion_point(module_scope)



import logging
from typing import Optional, List

from google.protobuf.empty_pb2 import Empty

from .crypto import PrivateKey
from .util import create_deploy_data

from .pb.DeployService_pb2 import (DataAtNameQuery, ListeningNameDataResponse as Data)
from .pb.DeployService_pb2_grpc import (DeployServiceStub)
from .pb.ProposeService_pb2_grpc import (ProposeServiceStub)
from .pb.RhoTypes_pb2 import (Par, Expr, GUnforgeable, GDeployId)


class RClientException(Exception):

    def _init__(self, message):
        super().__init__(message)


class DataQueries:

    @staticmethod
    def public_names(names: List[str]) -> Par:
        exprs = [Expr(g_string=n) for n in names]
        return Par(exprs=exprs)

    @staticmethod
    def deploy_id(deploy_id: bytes) -> Par:
        g_deploy_id = GDeployId(sig=deploy_id)
        g_unforgeable = GUnforgeable(g_deploy_id_body=g_deploy_id)
        return Par(unforgeables=[g_unforgeable])


class RClient:

    def __init__(self, channel):
        self.channel = channel

    def _check_response(self, response):
        logging.debug('gRPC response: %s', str(response))
        if response.HasField('error'):
            raise RClientException('\n'.join(response.error.messages))

    def deploy(
        self,
        key: PrivateKey,
        term: str,
        phlo_price: int,
        phlo_limit: int,
        valid_after_block_no: int = -1,
        timestamp_millis: int = -1,
    ) -> bytes:
        deploy_data = create_deploy_data(
            key, term, phlo_price, phlo_limit, valid_after_block_no, timestamp_millis
        )
        stub = DeployServiceStub(self.channel)
        response = stub.DoDeploy(deploy_data)
        self._check_response(response)
        return deploy_data.sig

    def propose(self):
        stub = ProposeServiceStub(self.channel)
        response = stub.propose(Empty())
        self._check_response(response)

    def get_data_at_name(self, par: Par, depth: int = -1) -> Optional[Data]:
        query = DataAtNameQuery(depth=depth, name=par)
        stub = DeployServiceStub(self.channel)
        response = stub.listenForDataAtName(query)
        self._check_response(response)
        data = None
        wrapped = response.success.response
        if hasattr(wrapped, 'value'):
            data = Data.FromString(wrapped.value)
        return data

    def get_data_at_public_names(self, names: List[str], depth: int = -1) -> Optional[Data]:
        return self.get_data_at_name(DataQueries.public_names(names), depth)

    def get_data_at_deploy_id(self, deploy_id: bytes, depth: int = -1) -> Optional[Data]:
        return self.get_data_at_name(DataQueries.deploy_id(deploy_id), depth)



import unittest
from ecdsa.keys import BadSignatureError

from .crypto import blake2b_32, PrivateKey, PublicKey


class TestCrypto(unittest.TestCase):

    def test_blake2b_32(self):
        in_out_list = [
            ('', '0e5751c026e543b2e8ab2eb06099daa1d1e5df47778f7787faab45cdf12fe3a8'),
            ('foo', 'b8fe9f7f6255a6fa08f668ab632a8d081ad87983c77cd274e48ce450f0b349fd'),
            (
                'sRHyrQZuEdI5WyvJD1YwCVaUpFEHCCZXF7HIIPMUIABwwr6EWNPRB1s3XkYzobM8TxHA2NHpxjzUzIxk',
                'cb44058e45aeca88339b16c53d3e2544246bc9a6aa4f831cf2099d70037aba61'
            ),
        ]
        for input_str, expected_output in in_out_list:
            actual_output = blake2b_32(input_str.encode()).digest().hex()
            self.assertEqual(expected_output, actual_output)

    identities = [
        (
            '1000000000000000000000000000000000000000000000000000000000000000',
            '0408ea9666139527a8c1dd94ce4f071fd23c8b350c5a4bb33748c4ba111faccae0620efabbc8ee2782e24e7c0cfb95c5d735b783be9cf0f8e955af34a30e62b945',
            '1111cnoFDAa7GubxBMHpPLbbediPegnjSdZwNjxg9oqYvSvSmfqQL'
        ),
        (
            '1111111111111111111111111111111111111111111111111111111111111111',
            '044f355bdcb7cc0af728ef3cceb9615d90684bb5b2ca5f859ab0f0b704075871aa385b6b1b8ead809ca67454d9683fcf2ba03456d6fe2c4abe2b07f0fbdbb2f1c1',
            '111125UED2qbF3aman9PYvfeTk81TuP4fVL5nYsEXtFyTtaLPcHW61'
        ),
        (
            '2222222222222222222222222222222222222222222222222222222222222222',
            '04466d7fcae563e5cb09a0d1870bb580344804617879a14949cf22285f1bae3f276728176c3c6431f8eeda4538dc37c865e2784f3a9e77d044f33e407797e1278a',
            '1111yoPaoSxvmNRGDpP9GUGToHaCdUFEu3KdbNgTuyydUgg6cty7g'
        )
    ]

    def test_privatekey_from_hex(self):
        for key_hex, expected_pub_key_hex, _ in self.identities:
            key = PrivateKey.from_hex(key_hex)
            pub_key = key.get_public_key()
            actual_pub_key_hex = pub_key.to_bytes().hex()
            self.assertEqual(expected_pub_key_hex, actual_pub_key_hex)

    def test_publickey_from_hex(self):
        for key_hex, pub_key_hex, _ in self.identities:
            key = PrivateKey.from_hex(key_hex)
            pub_key = PublicKey.from_hex(pub_key_hex)
            expected_pub_key_bytes = key.get_public_key().to_bytes()
            actual_pub_key_bytes = pub_key.to_bytes()
            self.assertEqual(expected_pub_key_bytes, actual_pub_key_bytes)

    def test_key_to_hex(self):
        for key_hex, pub_key_hex, _ in self.identities:
            private_key = PrivateKey.from_hex(key_hex)
            self.assertEqual(private_key.to_hex(), key_hex)
            public_key = PublicKey.from_hex(pub_key_hex)
            self.assertEqual(public_key.to_hex(), pub_key_hex)

    def test_sign_verify_good(self):
        for key_hex, pub_key_hex, _ in self.identities:
            key = PrivateKey.from_hex(key_hex)
            pub_key = PublicKey.from_hex(pub_key_hex)
            message = 'hello rchain'.encode()
            signature = key.sign(message)
            pub_key.verify(signature, message)

    def test_sign_verify_bad(self):
        for key_hex, pub_key_hex, _ in self.identities:
            key = PrivateKey.from_hex(key_hex)
            pub_key = PublicKey.from_hex(pub_key_hex)
            sign_message = 'hello rchain'.encode()
            verify_message = 'hello world'.encode()
            signature = key.sign(sign_message)
            with self.assertRaises(BadSignatureError):
                pub_key.verify(signature, verify_message)

    def test_sign_block_hash_verify_good(self):
        for key_hex, pub_key_hex, _ in self.identities:
            key = PrivateKey.from_hex(key_hex)
            pub_key = PublicKey.from_hex(pub_key_hex)
            message = 'hello rchain'.encode()
            digest = blake2b_32(message).digest()
            signature = key.sign_block_hash(digest)
            pub_key.verify_block_hash(signature, digest)

    def test_sign_block_hash_verify_bad(self):
        for key_hex, pub_key_hex, _ in self.identities:
            key = PrivateKey.from_hex(key_hex)
            pub_key = PublicKey.from_hex(pub_key_hex)
            sign_message = 'hello rchain'.encode()
            verify_message = 'hello world'.encode()
            block_hash = blake2b_32(sign_message).digest()
            signature = key.sign_block_hash(block_hash)
            verify_digest = blake2b_32(verify_message).digest()
            with self.assertRaises(BadSignatureError):
                pub_key.verify_block_hash(signature, verify_digest)

    def test_address(self):
        for _, pub_key_hex, expected_address in self.identities:
            pub_key = PublicKey.from_hex(pub_key_hex)
            actual_address = pub_key.get_rev_address()
            self.assertEqual(expected_address, actual_address)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Either.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Either.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\n\022coop.rchain.either'),
  serialized_pb=_b('\n\x0c\x45ither.proto\x1a\x15scalapb/scalapb.proto\x1a\x19google/protobuf/any.proto\",\n\tEitherAny\x12\x10\n\x08type_url\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\"\x1f\n\x0b\x45itherError\x12\x10\n\x08messages\x18\x01 \x03(\t\"-\n\rEitherSuccess\x12\x1c\n\x08response\x18\x01 \x01(\x0b\x32\n.EitherAny\"U\n\x06\x45ither\x12\x1d\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x0c.EitherErrorH\x00\x12!\n\x07success\x18\x02 \x01(\x0b\x32\x0e.EitherSuccessH\x00\x42\t\n\x07\x63ontentB\x14\n\x12\x63oop.rchain.eitherb\x06proto3')
  ,
  dependencies=[scalapb_dot_scalapb__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_EITHERANY = _descriptor.Descriptor(
  name='EitherAny',
  full_name='EitherAny',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type_url', full_name='EitherAny.type_url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='EitherAny.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=110,
)


_EITHERERROR = _descriptor.Descriptor(
  name='EitherError',
  full_name='EitherError',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='messages', full_name='EitherError.messages', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=112,
  serialized_end=143,
)


_EITHERSUCCESS = _descriptor.Descriptor(
  name='EitherSuccess',
  full_name='EitherSuccess',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='response', full_name='EitherSuccess.response', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=145,
  serialized_end=190,
)


_EITHER = _descriptor.Descriptor(
  name='Either',
  full_name='Either',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error', full_name='Either.error', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='success', full_name='Either.success', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='content', full_name='Either.content',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=192,
  serialized_end=277,
)

_EITHERSUCCESS.fields_by_name['response'].message_type = _EITHERANY
_EITHER.fields_by_name['error'].message_type = _EITHERERROR
_EITHER.fields_by_name['success'].message_type = _EITHERSUCCESS
_EITHER.oneofs_by_name['content'].fields.append(
  _EITHER.fields_by_name['error'])
_EITHER.fields_by_name['error'].containing_oneof = _EITHER.oneofs_by_name['content']
_EITHER.oneofs_by_name['content'].fields.append(
  _EITHER.fields_by_name['success'])
_EITHER.fields_by_name['success'].containing_oneof = _EITHER.oneofs_by_name['content']
DESCRIPTOR.message_types_by_name['EitherAny'] = _EITHERANY
DESCRIPTOR.message_types_by_name['EitherError'] = _EITHERERROR
DESCRIPTOR.message_types_by_name['EitherSuccess'] = _EITHERSUCCESS
DESCRIPTOR.message_types_by_name['Either'] = _EITHER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EitherAny = _reflection.GeneratedProtocolMessageType('EitherAny', (_message.Message,), {
  'DESCRIPTOR' : _EITHERANY,
  '__module__' : 'Either_pb2'
  # @@protoc_insertion_point(class_scope:EitherAny)
  })
_sym_db.RegisterMessage(EitherAny)

EitherError = _reflection.GeneratedProtocolMessageType('EitherError', (_message.Message,), {
  'DESCRIPTOR' : _EITHERERROR,
  '__module__' : 'Either_pb2'
  # @@protoc_insertion_point(class_scope:EitherError)
  })
_sym_db.RegisterMessage(EitherError)

EitherSuccess = _reflection.GeneratedProtocolMessageType('EitherSuccess', (_message.Message,), {
  'DESCRIPTOR' : _EITHERSUCCESS,
  '__module__' : 'Either_pb2'
  # @@protoc_insertion_point(class_scope:EitherSuccess)
  })
_sym_db.RegisterMessage(EitherSuccess)

Either = _reflection.GeneratedProtocolMessageType('Either', (_message.Message,), {
  'DESCRIPTOR' : _EITHER,
  '__module__' : 'Either_pb2'
  # @@protoc_insertion_point(class_scope:Either)
  })
_sym_db.RegisterMessage(Either)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
